#ifndef _XC3S_Initiator_H
#define _XC3S_Initiator_H

#include "Prefix.h"
#include "XC3S_Register.h"
#include "EARTH_PT.h"

namespace EARTH {
namespace XC3S {
	class Register;
	class Buffer;

	class Initiator {
	public:
		Initiator(XC3S::Register *register_) : mRegister(register_)
		{
			_ASSERTE(mRegister);

			for (uint i=0; i<sizeof(mStreamEnable)/sizeof(*mStreamEnable); i++) {
				mStreamEnable[i] = false;
				mStreamGray[i] = 0;
			}
		}

		~Initiator()
		{
		}

		uint DataWhenClose() const
		{
			uint data = BIT_1(1) << BIT_CONTROL_END | BIT_1(1) << BIT_CONTROL_END_;

			return data;
		}

		status SetStreamEnable(uint tunerIndex, PT::Device::ISDB isdb, bool enable)
		{
			uint index = tunerIndex*2 + isdb;
			uint bit = enable ? BIT_1(1) : BIT_1(0);
			uint data = BIT_1(bit) << (BIT_TS_ENABLE + index) | BIT_1(1) << (BIT_TS_ENABLE_ + index);

			mRegister->Write4(A_TS_TEST_ENABLE, data);

			mStreamEnable[index] = enable;

			return PT::STATUS_OK;
		}

		status GetStreamEnable(uint tunerIndex, PT::Device::ISDB isdb, bool *enable) const
		{
			if (enable == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			uint index = tunerIndex*2 + isdb;
			*enable = mStreamEnable[index];

			return PT::STATUS_OK;
		}

		status SetStreamGray(uint tunerIndex, PT::Device::ISDB isdb, uint gray)
		{
			if ((1<<(3)) <= gray) return PT::STATUS_INVALID_PARAM_ERROR;

			uint index = tunerIndex*2 + isdb;
			uint data = (BIT_1(1)<<3 | gray) << (4*index);

			mStreamGray[index] = gray;

			mRegister->Write4(A_TS_GRAY, data);

			return PT::STATUS_OK;
		}

		status GetStreamGray(uint tunerIndex, PT::Device::ISDB isdb, uint *gray) const
		{
			if (gray == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			uint index = tunerIndex*2 + isdb;
			*gray = mStreamGray[index];

			return PT::STATUS_OK;
		}

		void Reset()
		{
			mRegister->Write4(A_CONTROL, BIT_1(1) << BIT_CONTROL_RESET);
		}

		void Increment()
		{
			mRegister->Write4(A_CONTROL, BIT_1(1) << BIT_CONTROL_INCREMENT);
		}

		void SetTestMode(bool testMode)
		{
			uint bit = testMode ? BIT_1(1) : BIT_1(0);

			uint data = BIT_1(bit) << BIT_TS_TEST | BIT_1(1) << BIT_TS_TEST_;

			mRegister->Write4(A_TS_TEST_ENABLE, data);
		}

		void Begin(uint startAddress, bool enable_fast_back_to_back = false)
		{
			mRegister->Write4(A_ADDRESS, startAddress);

			uint bit = enable_fast_back_to_back ? BIT_1(1) : BIT_1(0);

			uint data =   BIT_1(1  ) << BIT_CONTROL_BEGIN
						| BIT_1(bit) << BIT_CONTROL_ENABLE_FAST_BACK_TO_BACK | BIT_1(1) << BIT_CONTROL_ENABLE_FAST_BACK_TO_BACK_
						| BIT_1(0  ) << BIT_CONTROL_END                      | BIT_1(1) << BIT_CONTROL_END_;

			mRegister->Write4(A_CONTROL, data);
		}

		void End()
		{
			uint data = BIT_1(1) << BIT_CONTROL_END | BIT_1(1) << BIT_CONTROL_END_;

			mRegister->Write4(A_CONTROL, data);
		}

	protected:
		enum {
			A_CONTROL        = 0x00,
			A_TS_TEST_ENABLE = 0x02,
			A_TS_GRAY        = 0x03,
			A_ADDRESS        = 0x05
		};

		enum {
			BIT_CONTROL_RESET = 4,
			BIT_CONTROL_INCREMENT,
			BIT_CONTROL_BEGIN,

			BIT_CONTROL_ENABLE_FAST_BACK_TO_BACK = 18,
			BIT_CONTROL_END,

			BIT_CONTROL_ENABLE_FAST_BACK_TO_BACK_ = BIT_CONTROL_ENABLE_FAST_BACK_TO_BACK + 8,
			BIT_CONTROL_END_                      = BIT_CONTROL_END                      + 8
		};

		enum {
			BIT_TS_ENABLE,
			BIT_TS_TEST = 16,

			BIT_TS_ENABLE_ = BIT_TS_ENABLE + 8,
			BIT_TS_TEST_   = BIT_TS_TEST   + 8
		};

		XC3S::Register *const mRegister;
		bool mStreamEnable[4];
		uint mStreamGray[4];
	};
}
}

#endif
