#ifndef _PT_Bus_H
#define _PT_Bus_H

#include "Prefix.h"
#include "EARTH_PT.h"
#include "PT_Device.h"
#include "PT_Device2.h"
#include <Windows.h>
#include "PCI_.h"
#include "Memory_.h"
#include "OS_Process.h"
#include "OS_Semaphore.h"
#include <vector>

namespace EARTH {
namespace Memory { class _; }
namespace PCI    { class _; }
namespace PT {

	class ROM_;
	class Device_;

	class Bus_ : public Bus {
	public:
		Bus_()
		{
			mPCI = NULL;
			mMemory = NULL;
			mSemaphore = NULL;
		}

		~Bus_()
		{
			delete_(mMemory);
			delete_(mPCI);
			delete_(mSemaphore);
		}

		status Init()
		{
			_ASSERTE(mSemaphore == NULL);
			mSemaphore = new OS::Semaphore(1);
			_ASSERTE(mSemaphore);

			HMODULE library = ::LoadLibrary(TEXT("wdapi1002.dll"));
			if (library == NULL) return STATUS_WDAPI_LOAD_ERROR;
			::FreeLibrary(library);
			library = NULL;

			bool isWow64 = OS::Process::IsWow64();
			uint kernelBit = isWow64 ? 64 : 32;

			_ASSERTE(mPCI == NULL);
			mPCI = static_cast<PCI::_ *>(new PCI::__(kernelBit));
			_ASSERTE(mPCI);
			
			status status;

			status = mPCI->Init();
			if (status) return status;

			HANDLE driver = mPCI->Bus()->Handle();
			_ASSERTE(driver);

			Memory::_::argument argument;
			argument.driver = driver;
			argument.kernelBit = kernelBit;

			_ASSERTE(mMemory == NULL);
			mMemory = static_cast<class Memory::_ *>(new Memory::__(&argument));
			_ASSERTE(mMemory);

			return status;
		}

		virtual status Delete()
		{
			if (mDevice.size() != 0) return STATUS_ALL_DEVICES_MUST_BE_DELETED_ERROR;
			delete this;
			return STATUS_OK;
		}

		virtual status GetVersion(uint *version) const
		{
			if (version == NULL) return STATUS_INVALID_PARAM_ERROR;

			*version = 0x200;

			return STATUS_OK;
		}

		virtual status Scan(DeviceInfo *deviceInfoPtr, uint *deviceInfoCount, uint maxBadBitCount)
		{
			if ((deviceInfoPtr == NULL) || (deviceInfoCount == NULL)) return STATUS_INVALID_PARAM_ERROR;
			if (3 < maxBadBitCount) return STATUS_INVALID_PARAM_ERROR;

			if (mPCI == NULL) return STATUS_INTERNAL_ERROR;
			status status = mPCI->Bus()->ScanDevice(deviceInfoPtr, deviceInfoCount, maxBadBitCount);

			return status;
		}

		virtual status NewDevice(const DeviceInfo *deviceInfoPtr, Device **device_, Device2 **device2_)
		{
			if ((deviceInfoPtr == NULL) || (device_ == NULL)) return STATUS_INVALID_PARAM_ERROR;

			#if !defined(_DEBUG)
			if (device2_ != NULL) return STATUS_INVALID_PARAM_ERROR;
			#endif

			if ((mPCI == NULL) || (mMemory == NULL)) return STATUS_INTERNAL_ERROR;

			Device_ *device = new Device_(DeleteDevice_, this, mPCI, mMemory, deviceInfoPtr);
			*device_ = device;

			mSemaphore->Acquire();
			mDevice.push_back(device);
			mSemaphore->Release();

			#if defined(_DEBUG)
			*device2_ = device->Device2();
			#endif

			return STATUS_OK;
		}

		void DeleteDevice(Device_ *device)
		{
			mSemaphore->Acquire();

			for (std::vector<Device_ *>::iterator i=mDevice.begin(); i!=mDevice.end(); i++) {
				if (*i == device) {
					mDevice.erase(i);
					break;
				}
			}
			mSemaphore->Release();
		}

		static void DeleteDevice_(Device_ *device, void *argument)
		{
			Bus_ *that = static_cast<Bus_ *>(argument);
			_ASSERTE(that);
			that->DeleteDevice(device);
		}

	protected:
		Memory::_ *mMemory;
		PCI::_ *mPCI;
		OS::Semaphore *mSemaphore;
		std::vector<Device_ *> mDevice;
	};

}
}

#endif
