#ifndef _PCI_Bus__H
#define _PCI_Bus__H

#include "PCI_Bus.h"
#include "EARTH_PT.h"
#include <windows.h>
#include "windrvr.h"

namespace EARTH {
namespace PCI {
namespace KERNEL_BIT {

	class Bus_ : public Bus {
	public:	
		Bus_() : mHandle(INVALID_HANDLE_VALUE)
		{
		}

		virtual ~Bus_()
		{
			if (mHandle != INVALID_HANDLE_VALUE) {
				WD_Close(mHandle);
				mHandle = INVALID_HANDLE_VALUE;
			}
		}

		virtual status Init()
		{
			const char *NAME[] = {
				"windrvr6_EARTHSOFT_PT2",
				"windrvr6_EARTHSOFT_PT1"
			};

			status status = PT::STATUS_GENERAL_ERROR;

			for (uint i=0; i<sizeof(NAME)/sizeof(*NAME); i++) {
				status = Init2(NAME[i]);
				if (status == PT::STATUS_OK) break;
			}

			return status;
		}

		virtual status ScanDevice(PT::Bus::DeviceInfo *deviceInfoPtr, uint *deviceInfoCount, uint maxBadBitCount)
		{
			_ASSERTE(deviceInfoPtr);
			_ASSERTE(deviceInfoCount);
			uint inDeviceInfoCount = *deviceInfoCount;
			*deviceInfoCount = 0;

			WD_PCI_SCAN_CARDS scan;
			BZERO(scan);

			scan.searchId.dwVendorId = 0;
			scan.searchId.dwDeviceId = 0;
			scan.dwOptions           = WD_PCI_SCAN_DEFAULT;

			DWORD status = WD_PciScanCards(mHandle, &scan);
			if (status != WD_STATUS_SUCCESS) return PT::STATUS_WD_PciScanCards_ERROR;

			uint dstIndex = 0;
			for (uint i=0; i<scan.dwCards && dstIndex<inDeviceInfoCount; i++) {
				WD_PCI_SLOT slot = scan.cardSlot[i];

				WD_PCI_CONFIG_DUMP config;
				BZERO(config);

				uint sub_id;
	
				config.pciSlot  = slot;
				config.pBuffer  = &sub_id;
				config.dwOffset = 0x2c;
				config.dwBytes  = 4;
				config.fIsRead  = TRUE;
	
				DWORD status = WD_PciConfigDump(mHandle, &config);
				if ((status == WD_STATUS_SUCCESS) && (config.dwResult == PCI_ACCESS_OK)) {
					const ushort VENDOR_ID   = 0x10ee;
					const ushort VENDOR_ID_N = static_cast<ushort>(~VENDOR_ID);

					uint minBadCount = static_cast<uint>(-1);
					uint minBadIndex = 0;
					for (uint j=0; j<2; j++) {
						const ushort DEVICE_ID[2] = { 0x211a, 0x222a };
						ushort deviceId  =  DEVICE_ID[j];
						ushort deviceIdN = ~DEVICE_ID[j];
						
						uint badCount = 0;
						badCount += CountBadBits(scan.cardId[i].dwVendorId << 16 | scan.cardId[i].dwDeviceId, VENDOR_ID << 16 | deviceId);
						badCount += CountBadBits(sub_id, deviceIdN << 16 | VENDOR_ID_N);
						if (badCount < minBadCount) {
							minBadCount = badCount;
							minBadIndex = j;
						}
					}

					if (minBadCount <= maxBadBitCount) {
						deviceInfoPtr->Bus         = slot.dwBus;
						deviceInfoPtr->Slot        = slot.dwSlot;
						deviceInfoPtr->Function    = slot.dwFunction;
						deviceInfoPtr->PTn         = 1 + minBadIndex;
						deviceInfoPtr->BadBitCount = minBadCount;

						deviceInfoPtr++;
						dstIndex++;
						(*deviceInfoCount)++;
					}
				}
			}

			return PT::STATUS_OK;
		}

		virtual status WriteConfig(uint bus, uint slot, uint function, uint address, const void *data, uint size)
		{
			WD_PCI_SLOT pciSlot;
			pciSlot.dwBus      = bus;
			pciSlot.dwSlot     = slot;
			pciSlot.dwFunction = function;

			WD_PCI_CONFIG_DUMP config;
			BZERO(config);
			config.pciSlot  = pciSlot;
			config.pBuffer  = const_cast<void *>(data);
			config.dwOffset = address;
			config.dwBytes  = size;
			config.fIsRead  = FALSE;

			DWORD status = WD_PciConfigDump(mHandle, &config);
			if ((status != WD_STATUS_SUCCESS) || (config.dwResult != PCI_ACCESS_OK)) return PT::STATUS_WD_PciConfigDump_ERROR;

			return PT::STATUS_OK;
		}

		virtual status ReadConfig(uint bus, uint slot, uint function, uint address, void *data, uint size)
		{
			WD_PCI_SLOT pciSlot;
			pciSlot.dwBus      = bus;
			pciSlot.dwSlot     = slot;
			pciSlot.dwFunction = function;

			WD_PCI_CONFIG_DUMP config;
			BZERO(config);
			config.pciSlot  = pciSlot;
			config.pBuffer  = data;
			config.dwOffset = address;
			config.dwBytes  = size;
			config.fIsRead  = TRUE;

			DWORD status = WD_PciConfigDump(mHandle, &config);
			if ((status != WD_STATUS_SUCCESS) || (config.dwResult != PCI_ACCESS_OK)) return PT::STATUS_WD_PciConfigDump_ERROR;

			return PT::STATUS_OK;
		}

		virtual HANDLE Handle() const
		{
			return mHandle;
		}

	protected:
		HANDLE mHandle;

		status Init2(const char *driverName)
		{
			const char *name = WD_DriverName(driverName);
			if (name == NULL) return PT::STATUS_WD_DriverName_ERROR;

			_ASSERTE(mHandle == INVALID_HANDLE_VALUE);
			mHandle = WD_Open();
			if (mHandle == INVALID_HANDLE_VALUE) return PT::STATUS_WD_Open_ERROR;
	
			status status;

			WD_VERSION version;
			BZERO(version);

			status = WD_Version(mHandle, &version);
			if ((status != WD_STATUS_SUCCESS) || (version.dwVer != WD_VER)) {
				WD_Close(mHandle);
				mHandle = INVALID_HANDLE_VALUE;
				return PT::STATUS_WD_Version_ERROR;
			}

			WD_LICENSE license;
			BZERO(license);

			::strcpy_s(license.cLicense, /* \[XR[hJR: CZXR[hł邽 */);
			status = WD_License(mHandle, &license);
			if (status != WD_STATUS_SUCCESS) {
				WD_Close(mHandle);
				mHandle = INVALID_HANDLE_VALUE;
				return PT::STATUS_WD_License_ERROR;
			}

			return PT::STATUS_OK;
		}

		uint CountBadBits(uint a, uint b) const
		{
			uint c = a ^ b;

			uint count = 0;
			for (uint i=0; i<8*sizeof(c); i++) {
				if (BIT_SHIFT_MASK(c, i, 1)) {
					count++;
				}
			}

			return count;
		}
	};

	status NewBus(Bus **bus_)
	{
		Bus *bus = new Bus_;
	
		status status = bus->Init();
		if (status) {
			delete_(bus);
		}

		_ASSERTE(bus_);
		*bus_ = bus;

		return status;
	}
}
}
}

#endif
